//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: MinMax.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//
/*
 * MinMax.java
 *
 * Created on November 21, 2001, 5:01 PM
 */

package siena.xml;

/** This class is used to store the minimum/maximum values. The values are updated through
 * the use of the adjust methods. Only integer or double values, not both, can be kept track of.
 *
 * @author Chris Corliss
 * @version 0.1
 */
class MinMax {

    /** This holds the minimum integer value
     */    
    private long min = Long.MAX_VALUE;
    /** This holds the maximum integer value
     */    
    private long max = Long.MIN_VALUE;
    
    /** This holds the minimum double value
     */    
    private double minDouble = Double.POSITIVE_INFINITY;
    /** This holds the maximum double value
     */    
    private double maxDouble = Double.NEGATIVE_INFINITY;
    
    /** This method is used to let the object know of a new value it should keep track of.
     * If the value is greater or less than the current min/max values the object will
     * replace the corresponding min/max value with the one passed in.
     * @param newValue New value to take into account for the min/max values.
     */    
    void adjust(long newValue) {
        if (newValue < min) {
            min = newValue;
        }
        if (newValue > max) {
            max = newValue;
        }
    }
    
    /** This method is used to let the object know of a new value it should keep track of.
     * If the value is greater or less than the current min/max values the object will
     * replace the corresponding min/max value with the one passed
     * @param newValue New value to take into account for the min/max values.
     */    
    void adjust(double newValue) {
        if (newValue < minDouble) {
            minDouble = newValue;
        }
        if (newValue > maxDouble) {
            maxDouble = newValue;
        }
    }
        
    /** This method returns true if the MinMAx class contains integer values and not doubles.
     * @return True if integers are used, otherwise false
     */    
    public boolean usesInts() {
        if (min != Long.MAX_VALUE || max != Long.MIN_VALUE) {
            return true;
        }
        return false;
    }
    
    /** Return the minimum value the object was update with
     * @return The minimum integer value the object was update with
     */    
    public long getIntMin() {
        return min;
    }
    
    /** Return the maximum value the object was updated with
     * @return The maximum integer value the object was updated with
     */    
    public long getIntMax() {
        return max;
    }
    
    /** This function returns the minimum douible value the object was updated with. This function can not be used in conjunction with the getIntMin function.
     * @return Returns a double containing the smallest valeu the object was updated with.
     */    
    public double getDoubleMin() {
        return minDouble;
    }
    
    /** This returns the maximum double value the object was update with. This method can not be used in conjunction with the getIntMax method.
     * @return Returns a double containing the maximum value the object was updated with.
     */    
    public double getDoubleMax() {
        return maxDouble;
    }
}
