//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: XMLClient.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//
/* Generated by Together */

package siena.xml;

import siena.Notification;
import siena.Notifiable;
import java.util.HashMap;


/**
 * This is a client based on the example ThinClient class in Siena.
 * It can be used as a front end to the XML translation process so
 * that a user of the class can simply ask it to publish and XML
 * notification and it will do the rest.
 * @author Chris Corliss
 * @version 0.1
 * @see XMLProcessor
 * @since 11-05-2001
 */
 
public class XMLClient extends siena.ThinClient implements Observer {

    /**
     * This is the reference to the XMLProcessor object that will 
     * translate the XML notification passed in through the publish
     * method and return a Siena Notification object.
     * @since 11-05-2001
     * @see XMLProcessor 
     */
    private XMLProcessor translater;
    /** These are the subscriptions which the XMLClient has recieved from application clients
     */    
    private HashMap subscriptions = new HashMap();
    /** This holds the name of the file containing the current XSchema being used by the client
     */    
    private String currentXSchema = null;

    /** This a generic constructor for the class.
     * @stereotype constructor
     * @since 11-07-2001
     * @param server the uri of the server to connect to
     * 	      (e.g., "senp://host.domain.net:7654")
     * @throws InvalidSenderException Is thrown when there is a problem connecting to the SIENA server
     */
    public XMLClient(String server) throws siena.comm.InvalidSenderException {
    	super(server);
        translater = new XMLProcessor();
    }

    /**
     * This method takes a XML notification and publishes it to the
     * Siena server.
     * @since 11-05-2001
     * @param xmlNotification This is the XML notification to be sent through the SIENA server network
     * @throws SienaException This is thrown if there are problems translating the XML notification or sending it to a SIENA server
     */
    public void publish(String xmlNotification) throws siena.SienaException {
        Notification notification = translater.translate(xmlNotification);     
//        System.out.println(notification);
        this.publish(notification);
    }

    /**
     * This method sets the stylesheet used by the XML notification.
     *
     * @param filename The name of the file that contains the XSchema
     * @throws IOException An IOException is thrown if processing the file containing the XSchema has a problem.
     */

    public void setStyleSheet(String filename) throws java.io.IOException {
        try {
            translater.setStyleSheet(filename);
        } catch (Exception e) {
            e.printStackTrace();
        }
        currentXSchema = filename;
    }

    /**
     * This method sets the user's rules.
     *
     * @see Rules#
     * @param filename The name of the file that contains the users rules.
     * @throws SienaException Thrown if a problem occurs when processing the user rules file.
     */

    public void setUserRules(String filename) throws siena.SienaException {
        translater.setUserRules(filename);
    }

    /** This method will clear the user rules and use only the default rules when procsesing XML notifications.
     */    
    public void clearUserRules() {
        translater.clearUserRules();
    }
    
    /** This method is used by the XMLSubscriptionHandler to notify the client
     * when an XML notification has been matched to the subscription this client
     * generated.
     * @param handler Reference to the XMLSubscriptionHandler that was being observered which issued the update.
     */

    public void update(XMLSubscriptionHandler handler){
        siena.Notifiable notifiable = handler.getNotifiable();
        Notification xmlNotification = new Notification();
        xmlNotification.putAttribute("xml", handler.getNotification());
        try {
            notifiable.notify(xmlNotification);
        } catch (Exception exception) {
            exception.printStackTrace();
        }
    }

    /**
     * This method is used to subscribe to the XML notifications.
     * @param n The notifiable object which should be notified when an XML Notification has been recieved that matches the subscription.
     * @param xpath A String containing the XPath subscription
     * @exception siena.SienaException This is thrown when there is a problem translating the XPath into SIENA subscriptions.
     */
    public void subscribe(String xpath, Notifiable n) throws siena.SienaException {
        Object test = subscriptions.get(xpath);
        XMLSubscriptionHandler handler = null;
        siena.Filter filters[] = null;

        // The subscription is already in the system therefore we can ignore this
        // request.
        if (test != null) {
            return;
        }

        try {
            handler = new XMLSubscriptionHandler(n);
            handler.setStylesheet(currentXSchema);            
            handler.attach(this);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }
 
        filters = handler.buildFilter(xpath);
        for (int i = 0; i < filters.length; i++) {
            super.subscribe(filters[i], handler);
        }
        
        subscriptions.put(xpath, handler);
    }

    /** This method unsubscribes and XPath from the SIENA servers.
     * @param xpath The XPath subscription to remove
     * @param n The notifiable object that was associated with the XPath
     * @throws SienaException This is thrown if an error occurs when trying to unsubscribe the subscription(s) from the SIENA server.
     */    
    public void unsubscribe(String xpath, Notifiable n) throws siena.SienaException {
        XMLSubscriptionHandler handler = (XMLSubscriptionHandler)subscriptions.get(xpath);
        siena.Filter filters[] = null;

        if (handler != null) {
            subscriptions.remove(xpath);
        }

        try {
            filters = handler.buildFilter(xpath);
        } catch (siena.SienaException exception) {
            /**
             * One condition that this exception would be thrown here is if the xpath given
             * for the subscription failed also and the client program ignored it and is
             * now trying to unsubscribe the faulty xpath. Create a new exception with a 
             * different explanation than the one coming from the buildFilter method to 
             * give users a more precise error.
             */
            throw new siena.SienaException("Invalid XPath to unsubscribe from.");
        }
            
        for (int i = 0; i < filters.length; i++) {
            this.unsubscribe(filters[i],handler);
        }
    }
}
