//
//  This file is part of SienaXML, an XML interface to the 
//  Siena wide-area event notification system.
//  See http://www.cs.colorado.edu/serl/siena/
//
//  Author: Chris Corliss <corliss@falcon.stortek.com>
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2002 University of Colorado
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//  
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//  
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
//  USA, or send email to serl@cs.colorado.edu.
//
//
// $Id: Rule.java,v 1.1.1.1 2002/06/07 22:42:37 jgiacomo Exp $
//
/* Generated by Together */

package siena.xml.rules;

import org.w3c.dom.Node;
import java.util.StringTokenizer;
import java.util.Vector;

/** This object represent the idea of a Rule which is applied to XML statements. Currently
 * the rule is used for XSchema processing only.
 */
public class Rule {

    /** If the different attribute conditions are joined by the boolean operation "and"
     */    
    private static final int ANDRULETYPE = 0;
    /** If the different attribute conditions are joined by the boolean operation "or"
     */    
    private static final int ORRULETYPE = 1;    

    /** Target XML tag the rule can be applied to
     */    
    private String targetTag = null;
    /** The XML tag path to get to the actual target of the rule
     */    
    private Vector targetChildren = new Vector();
    /** Target attribute of the XML tag the rule is targetted for.
     */    
    private String targetAttribute = null;
    /** List of attribute targets of the XML tag
     */    
    private String[] attributeTargets = new String[0];
    /** Conditions which need to be met before the rule can be applied to an XML DOM tree node.
     */    
    private java.util.Vector conditions = null;
    /** The action to be done when the rule is applied to a node.
     */    
    private String action = null;
    /** Type of rule this is: AND or OR.
     */    
    private int ruleType = ANDRULETYPE;

    /**
     * This is the constructor for the rule.
     *
     * @param target This is the target string to apply the rule to. This
     *               can be a tag or an attribute in the XSchema file.
     * @param action A string containing the action.
     */

    public Rule(String target, String action) {
        this.action = action;
        StringTokenizer tokenizer = new StringTokenizer(target, "@");

        conditions = new java.util.Vector();

        targetTag = tokenizer.nextToken();
        if (tokenizer.hasMoreTokens()) {
            targetAttribute = tokenizer.nextToken();
        }
        
        tokenizer = new StringTokenizer(targetTag,"/");
        targetTag = tokenizer.nextToken();        
        while (tokenizer.hasMoreTokens()) {
            targetChildren.addElement(tokenizer.nextToken());
        }
    }

    /**
     * This method will return the string version of the action.
     *
     * @return String containing the action.
     */

    public String getAction() {
        return action;
    }

    /**
     * This method determins if a DOM tree node matches the target and condition
     * for this rule. If there are no conditions then this method will alwyas
     * return true.
     *
     * @return true if the rule should be applied to this node, otherwsie false.
     * @param node The node to be tested to see if the rule can be applied to it.
     */

    public boolean doesRuleApplyToNode(Node node) {
        boolean result = false;
        boolean tempResult = false;        
        Condition condition = null;        
        Node testNode = null;

        if (!targetTag.equals("any") && !targetTag.equals(node.getLocalName())) {
            return false;
        }

        if (targetChildren.size() > 0) {
            testNode = getFinalTargetNode(node);
            if (testNode == null) {
                return false;
            }
        } else {
            testNode = node;
        }

        for (int i = 0; i < conditions.size(); i++) {
            condition = (Condition)conditions.elementAt(i);
            if (targetAttribute != null) {
                tempResult = condition.meetCondition(getAttributeNode(targetAttribute, testNode));
            } else if (attributeTargets.length > 0) {
                tempResult = condition.meetCondition(getAttributeNode(attributeTargets[i], testNode));
            } else {
                tempResult = condition.meetCondition(testNode);
            }
            
            if (!tempResult && ruleType == Rule.ANDRULETYPE) {
                return false;
            } else {
                result |= tempResult;
            }
        }

        return result;
    }
    
    /** Returns the actual XML target Node for the rule if a path was specified for the rules target.
     * @param node Parent node to start the search at
     * @return Node containing the XML tag the rule is target for, or null if a Node object is not found
     */    
    private Node getFinalTargetNode(Node node) {
        Node currentNode = node;
        for (int i = 0; i < targetChildren.size(); i++) {
            currentNode = siena.xml.DOMTreeProcessing.findChildByName((String)targetChildren.elementAt(i),currentNode);
            if (currentNode == null) {
                return null;
            }
        }    
        
        return currentNode;
    }
    
    /**
     * This method is used to get an attribute from the node using only the name
     * of the attribute to do it.
     *
     * @param name String containing the name of the attribute to get.
     * @param node The node to get the attribute from.
     * @return A Node for the attribute if it exists, otherwise null.
     */
    
    private Node getAttributeNode(String name, Node node) {
       org.w3c.dom.NamedNodeMap attributes = null;

        if (node.hasAttributes()) {
            attributes = node.getAttributes();
            if (attributes == null) {
                return null;
            }
            return (attributes.getNamedItem(name));
        } else {
            return null;
        }
    }
    
    /**
     * This method will add a condition to the list of conditions which need to
     * be met by the target in order for the rule to apply to it.
     *
     * @param condition A string description of the condition.
     */

    public void addCondition(Condition condition) {
        conditions.addElement(condition);
    }

    /**
     * This method returns the target string for the rule.
     *
     * @return String containing the target tag.
     */

    public String getTarget() {
        if (targetAttribute != null) {
            return targetTag+"@"+targetAttribute;
        } else {
            return targetTag;
        }
    }

    /** Sets this rule to an "and" rule
     */    
    public void setToAndRule() {
        ruleType = ANDRULETYPE;
    }

    /** Sets this rule to an "or" rule
     */    
    public void setToOrRule() {
        ruleType = ORRULETYPE;
    }

    /** Returns the XML target tag. If the target tag is a path, the first XML tag is returned.
     */    
    public String getTargetTag() {
        return targetTag;
    }

    /** Sets the attribute targets for the rule.
     */    
    public void setAttributeTargets(Vector targets) {
        attributeTargets = (String[])targets.toArray(attributeTargets);
        for (int i = 0; i < attributeTargets.length; i++) {
            attributeTargets[i] = attributeTargets[i].replace('@',' ').trim();
        }
    }
    
    /**
     * This method will apply the rule to the node. A string is returned that
     * is either the name of the SIENA notification attribute that the value
     * of the node should be stored in or a processing instruction for the node.
     *
     * @param node A DOM tree node for the XML document
     * @return A String containing a attribute name or processing instruction.
     */
     
    public String apply(Node node) {
        Node valueNode = null;
        Node attrNodeName = null;
        String nodeName = null;
        String storage = null;
        org.w3c.dom.NamedNodeMap attributes = null;
        StringTokenizer tokens = null;

        if (action.equals("ignore")) {
            return "IGNORE";
        }

        if (action.equals("path")) {
            return "PATH";
        }

        if (action.equals("novalue")) {
            return "NOVALUE";    
        }
        
        tokens = new StringTokenizer(action, ":");
        if (tokens.countTokens() != 1) {            
            tokens.nextToken();
        }
        
        storage = tokens.nextToken();
        
        if (node.hasAttributes()) {
            attributes = node.getAttributes();
            attrNodeName = attributes.getNamedItem("name");
            
            // The general case for when this method is being executed  is when 
            // the rule is being applied to a XSchema. However, when the user 
            // specifies his own rules then the rule will be for the xml and not
            // the XSchema therefore see if we can use the targetAttribute to 
            // get the attribute node of interest.
            
            if (targetAttribute != null && attrNodeName == null) {
                nodeName = targetAttribute;
            } 
            
            if (attrNodeName != null) {
                nodeName = attrNodeName.getNodeValue();
            }
        }
        
        // If the node is simply a tag with no value or attributes, ignore it.
        if (node.hasChildNodes()) {
            valueNode = node.getFirstChild();
            if (valueNode.getNodeType() != Node.TEXT_NODE) {
                valueNode = null;
            }
        }
        
        if (attributes == null && valueNode == null) {
            return null;
        }        
        
        if (action.startsWith("group") && !storage.equalsIgnoreCase("currenttag")) {
            return storage;
        }
        
        if (action.startsWith("unique")) {
            if (nodeName != null) {
                return (storage+"$"+nodeName);
            } else {
                // If nodeName is null then the uniqueness for the storage tag 
                // is the target tag or the last child of the target path 
                // of this rule.
                if (targetChildren.size() > 0) {
                    return(storage+"$"+targetChildren.elementAt(targetChildren.size()-1));
                } else {
                    return (storage+"$"+targetTag);
                }
            }
        }        

        if (storage.equals("currentTag")) {
            return nodeName;
        } else {
            return storage;
        }
    }

    /** Method to convert the Object to a string.
     * @return String representation of the Rule object.
     */    
    public String toString() {
        StringBuffer buffer = new StringBuffer("rule: ");
        buffer.append(targetTag);
        buffer.append(" ");
        buffer.append(targetAttribute);
        for (int i = 0; i < attributeTargets.length; i++) {
            buffer.append(" ");
            buffer.append(attributeTargets[i]);
        }
        buffer.append(" ");
        buffer.append(action);
        return buffer.toString();

    }
}

